/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: cache.c,v 1.13 2004/03/07 15:10:35 shtrom Exp $
 * binary cache handling function to avoid reparsing the timetable when not
 * necessary
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include "ouvej.h"

#define TTFNAME		"/timetable"
#define CACHENAME	"/.ouvej.cache"

/*
 * The magic number is (4+MAJOR)*16+2*MINOR
 */
int cachemagic=0x42;

int cache_check(char *homedir)
{
	char *ttpath, *cachepath;
	struct stat ttstat, cachestat;

	ttpath=timetable_path(homedir);
	cachepath=cache_path(homedir);

	if(stat(ttpath,&ttstat)<0) {
		ERROR("can't open %s (%s)\n", ttpath, strerror(errno));
		exit(1);
	}
	
	if(stat(cachepath,&cachestat)==0) {

		DPRINT("%s: %d\n",ttpath,(int)ttstat.st_mtime);
		DPRINT("%s: %d\n",cachepath,(int)cachestat.st_mtime);

		if((int)ttstat.st_mtime>(int)cachestat.st_mtime) {
			fprintf(stderr,"Timetable is newer than cache, regenerating...\n");
			cache_regen(ttpath, cachepath);
		}
	} else {
		fprintf(stderr,"Cachefile not found, regenerating...\n");
		cache_regen(ttpath, cachepath);
	}
	return 0;
}

FILE *_cache_ropen(char *cachename, int iter)
{
	FILE *fp;
	int filemagic=0;
	int waserr=0;

	fp=fopen(cachename,"r");
	if(!fp) {
		DPRINT("can't open cachefile for reading\n");
		waserr=1;
	}
	
	if(!waserr) {
		fread(&filemagic,sizeof(filemagic),1,fp);
		if(filemagic!=cachemagic) {
			DPRINT("magic number mismatch: expected 0x%x, got 0x%x\n",cachemagic,filemagic);
			fclose(fp);
			waserr=1;
		}
	}

	if(waserr) {
		/*
		 * if an error occured while trying to open or checking the
		 * magic number, we regenerate the cachefile and try to reopen
		 * it
		 *
		 * we don't want to go into an infinite loop, so we specify that
		 * if the openning still fails, we should stop (with iter)
		 */
		if(!iter) {
			DPRINT("error while opening, regenerating cache\n");
			cache_regen(timetable_path(get_homedir(NULL)),cachename);
			return _cache_ropen(cachename, 1);
		} else {
			DPRINT("\n");
			return NULL;
		}
	}
	
	DPRINT("openned cachefile (with magicnumber=0x%x)\n",filemagic);

	return fp;
}

FILE *cache_ropen(char *cachename)
{
	return _cache_ropen(cachename,0);
}

FILE *cache_wopen(char *cachename)
{
	FILE *fp;

	fp=fopen(cachename,"w");
	if(!fp) {
		DPRINT("can't open cachefile for writing\n");
		return NULL;
	}
	fwrite(&cachemagic,sizeof(cachemagic),1,fp);

	DPRINT("openned cachefile\n");

	return fp;
}

int cache_regen(char *ttpath, char *cachepath)
{
	ll_t *list;
	bt_t *btree;

	list=gen_list(ttpath);
	
	btree=btree_make(list);
	btree_write(btree, cachepath);

	return 0;
}

char *get_homedir(char **envp)
{
	static char *hd;

	/*
	 * we only generate the homedir path once
	 */
	if(hd)
		return hd;

	hd=getenv("HOME");
	if(!hd) {
		ERROR("$HOME not found, using . instead\n");
		return strdup(".");
	}
	
	DPRINT("$HOME is %s\n",hd);

	return hd;
}


char *timetable_path(char *homedir)
{
	static char *ttp;

	/*
	 * the path is only generated the first time it's needed
	 */
	if(ttp)
		return ttp;

	ttp=malloc(strlen(homedir)+sizeof(TTFNAME)+1);
	if(!ttp) {
		ERROR("couldn't allocate memory\n");
		exit(1);
	}

	*ttp=0;
	strcat(ttp,homedir);
	strcat(ttp,TTFNAME);

	DPRINT("timetable is %s\n",ttp);

	return ttp;
}

char *cache_path(char *homedir)
{
	static char *cp;

	/*
	 * the path is only generated the first time it's needed
	 */
	if(cp)
		return cp;

	cp=malloc(strlen(homedir)+sizeof(CACHENAME)+1);
	if(!cp) {
		ERROR("couldn't allocate memory\n");
		exit(1);
	}

	*cp=0;
	strcat(cp,homedir);
	strcat(cp,CACHENAME);

	DPRINT("cache is %s\n",cp);

	return cp;
}
